@@ -1,23 +1,10 @@
 /*
-   This file is part of the KDE libraries
-   Copyright (c) 2006, 2007 Thomas Braxton <kde.braxton@gmail.com>
-   Copyright (c) 1999 Preston Brown <pbrown@kde.org>
-   Copyright (c) 1997-1999 Matthias Kalle Dalheimer <kalle@kde.org>
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2006, 2007 Thomas Braxton <kde.braxton@gmail.com>
+    SPDX-FileCopyrightText: 1999 Preston Brown <pbrown@kde.org>
+    SPDX-FileCopyrightText: 1997-1999 Matthias Kalle Dalheimer <kalle@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "kconfig.h"
@@ -32,13 +19,13 @@
 #include "kconfigbackend_p.h"
 #include "kconfiggroup.h"
 
-#include <qcoreapplication.h>
-#include <qprocess.h>
-#include <qstandardpaths.h>
-#include <qbytearray.h>
-#include <qfile.h>
-#include <qlocale.h>
-#include <qdir.h>
+#include <QCoreApplication>
+#include <QProcess>
+#include <QStandardPaths>
+#include <QByteArray>
+#include <QFile>
+#include <QLocale>
+#include <QDir>
 #include <QProcess>
 #include <QSet>
 #include <QBasicMutex>
@@ -52,6 +39,9 @@
 
 bool KConfigPrivate::mappingsRegistered = false;
 
+// For caching purposes
+static bool s_wasTestModeEnabled = false;
+
 Q_GLOBAL_STATIC(QStringList, s_globalFiles) // For caching purposes.
 static QBasicMutex s_globalFilesMutex;
 Q_GLOBAL_STATIC_WITH_ARGS(QString, sGlobalFileName, (QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + QLatin1String("/kdeglobals")))
@@ -69,8 +59,16 @@ KConfigPrivate::KConfigPrivate(KConfig::OpenFlags flags,
       bFileImmutable(false), bForceGlobal(false), bSuppressGlobal(false),
       configState(KConfigBase::NoAccess)
 {
+    const bool isTestMode = QStandardPaths::isTestModeEnabled();
+    // If sGlobalFileName was initialised and testMode has been toggled,
+    // sGlobalFileName may need to be updated to point to the correct kdeglobals file
+    if (sGlobalFileName.exists() && s_wasTestModeEnabled != isTestMode) {
+        s_wasTestModeEnabled = isTestMode;
+        *sGlobalFileName = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + QLatin1String("/kdeglobals");
+    }
+
     static QBasicAtomicInt use_etc_kderc = Q_BASIC_ATOMIC_INITIALIZER(-1);
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
     if (use_etc_kderc.load() < 0) {
         use_etc_kderc.store( !qEnvironmentVariableIsSet("KDE_SKIP_KDERC"));    // for unit tests
     }
@@ -88,7 +86,7 @@ KConfigPrivate::KConfigPrivate(KConfig::OpenFlags flags,
             QStringLiteral("/etc/kde5rc");
 #endif
         if (!QFileInfo(etc_kderc).isReadable()) {
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
             use_etc_kderc.store(false);
 #else
             use_etc_kderc.storeRelaxed(false);
@@ -266,7 +264,7 @@ KConfig::KConfig(KConfigPrivate &d)
 KConfig::~KConfig()
 {
     Q_D(KConfig);
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
     if (d->bDirty && (d->mBackend && d->mBackend->ref.load() == 1)) {
 #else
     if (d->bDirty && (d->mBackend && d->mBackend->ref.loadRelaxed() == 1)) {
